import cv2
import numpy as np
from pallete_schema import PalleteSchema

class ComparisonComparison:
	def __init__ (self) :
		red = [0, 10, 200]
		green = [5, 250, 5]
		blue = [240, 0, 20]
		comp_colors = [red, green, blue]
		pallete = PalleteSchema('./palletes/test_pallete.json')
		colors = self.get_colors(pallete.colors)
		for cc in comp_colors :
			ccbgr = self.convert_color(cc, 'RGB', 'BGR')
			closest = self.closest(colors, ccbgr)
			print(f'{closest} for {ccbgr}')

	def get_colors (self, pallete) :
		colors = []
		for color in pallete :
			colors.append(color['color'])
		return colors

	def convert_color (self, color, color_space_a, color_space_b) :
		pixel = np.zeros([1, 1, 3], dtype=np.uint8)
		if color_space_a == 'RGB' :
			pixel = cv2.cvtColor(pixel, cv2.COLOR_BGR2RGB)
		elif color_space_a == 'LAB' :
			pixel = cv2.cvtColor(pixel, cv2.COLOR_BGR2LAB)
		elif color_space_a == 'HSV' :
			pixel = cv2.cvtColor(pixel, cv2.COLOR_BGR2HSV)
		#default is BGR
		pixel[:] = color
		if color_space_a == 'RGB' and color_space_b == 'BGR' :
			b = cv2.COLOR_RGB2BGR
		elif color_space_a == 'BGR' and color_space_b == 'RGB' :
			b = cv2.COLOR_BGR2RGB
		elif color_space_a == 'RGB' and color_space_b == 'LAB' :
			b = cv2.COLOR_RGB2LAB
		elif color_space_a == 'LAB' and color_space_b == 'RGB' :
			b = cv2.COLOR_LAB2RGB
		elif color_space_a == 'BGR' and color_space_b == 'LAB' :
			b = cv2.COLOR_BGR2LAB
		elif color_space_a == 'LAB' and color_space_b == 'BGR' :
			b = cv2.COLOR_LAB2BGR
		elif color_space_a == 'HSV' and color_space_b == 'LAB' :
			b = cv2.COLOR_HSV2LAB
		elif color_space_a == 'LAB' and color_space_b == 'HSV' :
			b = cv2.COLOR_LAB2HSV
		elif color_space_a == 'RGB' and color_space_b == 'HSV' :
			b = cv2.COLOR_RGB2HSV
		elif color_space_a == 'HSV' and color_space_b == 'RGB' :
			b = cv2.COLOR_HSV2RGB
		elif color_space_a == 'BGR' and color_space_b == 'HSV' :
			b = cv2.COLOR_BGRHSV
		elif color_space_a == 'HSV' and color_space_b == 'BGR' :
			b = cv2.COLOR_HSV2BGR

		cvt = cv2.cvtColor(pixel, b)
		return cvt[0, 0]

	def closest(self, colors, color):
		colors = np.array(colors)
		color = np.array(color)
		distances = np.sqrt(np.sum((colors - color) ** 2, axis=1))
		index_of_smallest = np.where(distances == np.amin(distances))
		smallest_distance = colors[index_of_smallest]
		return smallest_distance

if __name__ == "__main__":
	ComparisonComparison()