from sklearn.cluster import MiniBatchKMeans
import numpy as np
import argparse
import cv2

class Posterize:
	"""Posterize an image and then find nearest colors to use"""
	colors = []
	original_colors = []
	image = None
	pallete = None
	h = 0
	w = 0
	n_colors = 3
	white = [255, 255, 255]

	def __init__ (self, image, pallete, n_colors) :
		self.image = cv2.imread(image)
		(self.h, self.w) = self.image.shape[:2]
		self.pallete = pallete
		self.n_colors = n_colors + 1
		
		self.posterize()
		self.determine_colors()

	def posterize (self):
		lab = cv2.cvtColor(self.image, cv2.COLOR_BGR2LAB)
		feature = lab.reshape((self.h * self.w, 3))
		
		clusters = MiniBatchKMeans(n_clusters = self.n_colors, n_init = 'auto')
		
		labels = clusters.fit_predict(feature)
		quant = clusters.cluster_centers_.astype('uint8')[labels]
		
		rquant = quant.reshape((self.h, self.w, 3))
		rfeature = feature.reshape((self.h, self.w, 3))

		bgrquant = cv2.cvtColor(rquant, cv2.COLOR_LAB2BGR)
		#bgrfeature = cv2.cvtColor(rfeature, cv2.COLOR_LAB2BGR)

		self.image = bgrquant
		

		cv2.imshow("image", bgrquant)
		cv2.waitKey(0)
		cv2.destroyAllWindows()

	def determine_colors (self):
		reshaped = self.image.reshape(-1, self.image.shape[2])
		self.original_colors = np.unique(reshaped, axis=0)
		#print(self.original_colors)
		for i in range(self.n_colors) :
			mask = self.extract_color_mask(self.image, self.original_colors[i])
			cv2.imwrite(f'{i}.png', mask)

	def extract_color_mask (self, image, color):
		mask = cv2.inRange(image, color, color)
		return cv2.bitwise_not(mask)

	def convert_color (self, color, color_space_a, color_space_b) :
		pixel = np.zeros([1, 1, 3], dtype=np.uint8)
		if color_space_a == 'RGB' :
			pixel = cv2.cvtColor(pixel, cv2.COLOR_BGR2RGB)
		elif color_space_a == 'LAB' :
			pixel = cv2.cvtColor(pixel, cv2.COLOR_BGR2LAB)
		elif color_space_a == 'HSV' :
			pixel = cv2.cvtColor(pixel, cv2.COLOR_BGR2HSV)
		#default is BGR
		pixel[:] = color
		if color_space_a == 'RGB' and color_space_b == 'BGR' :
			b = cv2.COLOR_RGB2BGR
		elif color_space_a == 'BGR' and color_space_b == 'RGB' :
			b = cv2.COLOR_BGR2RGB
		elif color_space_a == 'RGB' and color_space_b == 'LAB' :
			b = cv2.COLOR_RGB2LAB
		elif color_space_a == 'LAB' and color_space_b == 'RGB' :
			b = cv2.COLOR_LAB2RGB
		elif color_space_a == 'BGR' and color_space_b == 'LAB' :
			b = cv2.COLOR_BGR2LAB
		elif color_space_a == 'LAB' and color_space_b == 'BGR' :
			b = cv2.COLOR_LAB2BGR
		elif color_space_a == 'HSV' and color_space_b == 'LAB' :
			b = cv2.COLOR_HSV2LAB
		elif color_space_a == 'LAB' and color_space_b == 'HSV' :
			b = cv2.COLOR_LAB2HSV
		elif color_space_a == 'RGB' and color_space_b == 'HSV' :
			b = cv2.COLOR_RGB2HSV
		elif color_space_a == 'HSV' and color_space_b == 'RGB' :
			b = cv2.COLOR_HSV2RGB
		elif color_space_a == 'BGR' and color_space_b == 'HSV' :
			b = cv2.COLOR_BGRHSV
		elif color_space_a == 'HSV' and color_space_b == 'BGR' :
			b = cv2.COLOR_HSV2BGR

		cvt = cv2.cvtColor(pixel, b)
		return cvt[0, 0]

	def closest(self, colors, color):
		colors = np.array(colors)
		color = np.array(color)
		distances = np.sqrt(np.sum((colors - color) ** 2, axis=1))
		index_of_smallest = np.where(distances == np.amin(distances))
		smallest_distance = colors[index_of_smallest]
		return smallest_distance 
